#include <Arduino.h>
#define min(a,b) ((a)<(b)?(a):(b))

#define ACC_SHAKE 10
#define ACC_STABLE 20
#define ACC_UNSTABLE 30

MPU6050 accelgyro;

int16_t ax, ay, az;
int16_t prev_ax, prev_ay, prev_az;
int16_t gx, gy, gz;

int prev_turn_state = -1;
int turns_acc = 0;
int turn_counter = 0;
long stability_start_tstamp = -1;
long shake_acc = 0;
long last_shake_measurement = 0;

int turn_states[TURN_STATE_COUNT][2] = {
  {0, 15000},
  {9600, 12500},
  {17300, -2500},
  {12500, -13200},
  {1200, -18500},
  { -10000, -14000},
  { -15600, -1100},
  { -11200, 10000},
};


void acc_setup() {
  Serial.println("Initializing I2C devices...");
  Wire.begin();
  accelgyro.initialize();

  // verify connection
  Serial.println("Testing device connections...");
  Serial.println(accelgyro.testConnection() ? "MPU6050 connection successful" : "MPU6050 connection failed");
}

int acc_get_stability() {
  int result;
  
  accelgyro.getAcceleration(&ax, &ay, &az);

  // shake detection
  if((millis() - last_shake_measurement) > SHAKE_MEASUREMENT_INTERVAL) {
    last_shake_measurement = millis();
    boolean valid_reading = (abs(ax) < 30000 && abs(ay) < 30000 && abs(az) < 30000);
    float force = valid_reading ? sqrt(ax * ax + ay * ay + az * az) : 0;
    shake_acc = shake_acc + abs(force - 15000) / 5 - 1500;
    if (shake_acc < 0) {
      shake_acc = 0;
    }
  }  
  if (shake_acc > SHAKE_THRESHOLD) {
    shake_acc = 0; // for next time
    DEBUG_PRINTLN("ACC_SHAKE");
    result = ACC_SHAKE;
  }
  else {
    if (abs(ax - prev_ax) < STABILITY_TOLERANCE &&
        abs(ay - prev_ay) < STABILITY_TOLERANCE &&
        abs(az - prev_az) < STABILITY_TOLERANCE) {
  
    if (stability_start_tstamp < 0) {
        stability_start_tstamp = millis();
      }
      else {
        if ((millis() - stability_start_tstamp) > STABILITY_QUALIFICATION_DURATION) {
          result = ACC_STABLE;
        }
        else {
          result = ACC_UNSTABLE;
        }
      }
    }
    else {
      stability_start_tstamp = -1;
      result = ACC_UNSTABLE;
    }
  }
  prev_ax = ax;
  prev_ay = ay;
  prev_az = az;
  return result;
}

int acc_get_turn_state() {
  int result = -1;
  //  boolean printout = false;
  for (int t = 0; t < TURN_STATE_COUNT; t++) {
    if (abs(ax - turn_states[t][0]) < SENSOR_TOLERANCE && abs(az - turn_states[t][1]) < SENSOR_TOLERANCE) {
      result = t;
    }
  }
  return result;
}


void acc_count_turns() {
  accelgyro.getAcceleration(&ax, &ay, &az);
  int turn_state = acc_get_turn_state();
  // are we in a valid turn state?
  if (turn_state >= 0) {
    // has there been a change in the turn state?
    if (turn_state != prev_turn_state) {
      // is this the second valid detection onwards?
      if (prev_turn_state > -1) {
        // is this a part of a smooth movement?
        int step_count =
          min(abs(turn_state - prev_turn_state),
              min(abs(turn_state - prev_turn_state + TURN_STATE_COUNT),
                  abs(turn_state - prev_turn_state - TURN_STATE_COUNT)));
        if (step_count <= MAX_TURN_SKIP) {
          // update accumulator
          turns_acc += step_count;
          // DEBUG_PRINTLN(turns_acc);
          if (abs(turns_acc) >= (TURN_STATE_COUNT)) {
            turn_counter++;
            DEBUG_PRINTLN(turn_counter);
            if (turns_acc > 0) {
              turns_acc -= TURN_STATE_COUNT;
            }
            else {
              turns_acc += TURN_STATE_COUNT;
            }
          }
        }
      }
      prev_turn_state = turn_state;
    }
  }
}

int acc_get_turn_counter() {
  return turn_counter;
}

int acc_reset_turn_counter() {
  turn_counter = 0;
}



