#define CLEAR_EEPROM_CREDENTIALS_PIN D6

#define CREDENTIALS_VALIDITY_FAIL 0
#define CREDENTIALS_VALIDITY_PASS 1
#define CREDENTIALS_VALIDITY_UNKNOWN 99

#define REG_STATUS_FAILED 0
#define REG_STATUS_PASSED 1
#define REG_STATUS_UNKNOWN 99

#define SSID_EEPROM_LOCATION 0
#define PASSPHRASE_EEPROM_LOCATION (SSID_EEPROM_LOCATION + sizeof(ssid))
#define CREDENTIALS_EXISTANCE_EEPROM_LOCATION (PASSPHRASE_EEPROM_LOCATION + sizeof(password))
#define CREDENTIALS_VALIDITY_EEPROM_LOCATION (CREDENTIALS_EXISTANCE_EEPROM_LOCATION + 3)

#define REGISTRATION_NAME_EEPROM_LOCATION (CREDENTIALS_VALIDITY_EEPROM_LOCATION + 3)
#define REGISTRATION_PHONE_EEPROM_LOCATION (REGISTRATION_NAME_EEPROM_LOCATION + sizeof(full_name))
#define REGISTRATION_STATUS_EEPROM_LOCATION (REGISTRATION_PHONE_EEPROM_LOCATION + sizeof(phone_number))

#define REPORT_HISTORY_EEPROM_LOCATION (REGISTRATION_STATUS_EEPROM_LOCATION + 3)

#define NEW_ROLL_FLAG_EEPROM_LOCATION (REPORT_HISTORY_EEPROM_LOCATION + 2*REPORT_HISTORY_LENGTH*sizeof(int))

/* Don't set this wifi credentials. They are configurated at runtime and stored on EEPROM */
char ssid[32 + 1] = "";
char password[32 + 1] = "";
char full_name[32 + 1] = "";
char phone_number[16 + 1] = "";
int history[REPORT_HISTORY_LENGTH * 2] = {-1};

char CODE_OK[2 + 1] = "OK";
char CODE_NOT_OK[2 + 1] = "NO";
char CODE_UNKNOWN[2 + 1] = "WH";

boolean eeprom_loadCredentials();
void eeprom_setCredentialsValidity(int validity);
void eeprom_setRegistrationStatus(int statu);
boolean eeprom_checkRegistrationExist();
boolean eeprom_checkCredentialsExist();
int* eeprom_getStoredReports();

/** Store WLAN credentials to EEPROM */
void eeprom_saveCredentials() {
  DEBUG_PRINTLN("eeprom_saveCredentials");
  EEPROM.begin(512);
  EEPROM.put(SSID_EEPROM_LOCATION, ssid);
  EEPROM.put(PASSPHRASE_EEPROM_LOCATION, password);
  EEPROM.put(CREDENTIALS_EXISTANCE_EEPROM_LOCATION, CODE_OK);
  EEPROM.commit();
  EEPROM.end();
}

/** Load WLAN credentials from EEPROM */
boolean eeprom_loadCredentials() {
  EEPROM.begin(512);
  EEPROM.get(SSID_EEPROM_LOCATION, ssid);
  EEPROM.get(PASSPHRASE_EEPROM_LOCATION, password);
  char ok[2 + 1];
  EEPROM.get(CREDENTIALS_EXISTANCE_EEPROM_LOCATION, ok);
  EEPROM.end();
  if (String(ok) != String(CODE_OK)) {
    ssid[0] = 0;
    password[0] = 0;
    return false;
  }
  else {
    DEBUG_PRINTLN("Recovered credentials:");
    DEBUG_PRINTLN(ssid);
    DEBUG_PRINTLN(strlen(password) > 0 ? "********" : "<no password>");
    return true;
  }
}


boolean eeprom_checkCredentialsExist() {
  EEPROM.begin(512);
  char exist[2 + 1];
  EEPROM.get(CREDENTIALS_EXISTANCE_EEPROM_LOCATION, exist);
  EEPROM.end();
  return (String(exist) == String(CODE_OK));
}

void eeprom_clear(boolean withEraseHistory) {
  Serial.println("Resetting EEPROM data");
  EEPROM.begin(512);
  EEPROM.put(CREDENTIALS_EXISTANCE_EEPROM_LOCATION, CODE_NOT_OK);
  if(withEraseHistory) {
    DEBUG_PRINTLN("Earsing history");
    for(int i = 0; i < REPORT_HISTORY_LENGTH * 2; ++i) history[i] = -1;
    EEPROM.put(REPORT_HISTORY_EEPROM_LOCATION, history);
  }
  EEPROM.put(NEW_ROLL_FLAG_EEPROM_LOCATION, 0);
  EEPROM.commit();
  EEPROM.end();
  eeprom_setCredentialsValidity(CREDENTIALS_VALIDITY_UNKNOWN);
  eeprom_setRegistrationStatus(REG_STATUS_UNKNOWN);
}

void eeprom_clear() {
  eeprom_clear(false);
}

void eeprom_setCredentialsValidity(int validity) {
  DEBUG_PRINT("setting credentials validity to: "); DEBUG_PRINTLN(validity);
  EEPROM.begin(512);
  switch (validity) {
    case CREDENTIALS_VALIDITY_PASS:
      EEPROM.put(CREDENTIALS_VALIDITY_EEPROM_LOCATION, CODE_OK);
      break;
    case CREDENTIALS_VALIDITY_FAIL:
      EEPROM.put(CREDENTIALS_VALIDITY_EEPROM_LOCATION, CODE_NOT_OK);
      break;
    default:
      EEPROM.put(CREDENTIALS_VALIDITY_EEPROM_LOCATION, CODE_UNKNOWN);
      break;
  }
  EEPROM.commit();
  EEPROM.end();
}

int eeprom_getCredentialsValidity() {
  DEBUG_PRINT("eeprom_getCredentialsValidity(): ");
  EEPROM.begin(512);
  char validity[2 + 1];
  // the 3 stands for length of OK string signaling credentials existance
  EEPROM.get(CREDENTIALS_VALIDITY_EEPROM_LOCATION, validity);
  EEPROM.end();
  String v = String(validity);
  if (v == String(CODE_OK)) {
    DEBUG_PRINTLN("CREDENTIALS_VALIDITY_PASS");
    return CREDENTIALS_VALIDITY_PASS;
  }
  else if (v == String(CODE_NOT_OK)) {
    DEBUG_PRINTLN("CREDENTIALS_VALIDITY_FAIL");
    return CREDENTIALS_VALIDITY_FAIL;
  }
  else {
    DEBUG_PRINTLN("CREDENTIALS_VALIDITY_UNKNOWN");
    return CREDENTIALS_VALIDITY_UNKNOWN;
  }
}

void eeprom_saveRegistration() {
  DEBUG_PRINTLN("eeprom_saveRegistration()");
  EEPROM.begin(512);
  EEPROM.put(REGISTRATION_NAME_EEPROM_LOCATION, full_name);
  EEPROM.put(REGISTRATION_PHONE_EEPROM_LOCATION, phone_number);
  EEPROM.put(REGISTRATION_STATUS_EEPROM_LOCATION, CODE_UNKNOWN);
  EEPROM.commit();
  EEPROM.end();
}

void eeprom_setRegistrationStatus(int statu) {
  EEPROM.begin(512);
  switch (statu) {
    case REG_STATUS_FAILED:
      EEPROM.put(REGISTRATION_STATUS_EEPROM_LOCATION, CODE_NOT_OK);
      break;
    case REG_STATUS_PASSED:
      EEPROM.put(REGISTRATION_STATUS_EEPROM_LOCATION, CODE_OK);
      break;
    case REG_STATUS_UNKNOWN:
      EEPROM.put(REGISTRATION_STATUS_EEPROM_LOCATION, CODE_UNKNOWN);
      break;
  }
  EEPROM.commit();
  EEPROM.end();
}

boolean eeprom_loadRegistration() {
  EEPROM.begin(512);
  EEPROM.get(REGISTRATION_NAME_EEPROM_LOCATION, full_name);
  EEPROM.get(REGISTRATION_PHONE_EEPROM_LOCATION, phone_number);
  EEPROM.end();
  if (!eeprom_checkRegistrationExist()) {
    full_name[0] = 0;
    phone_number[0] = 0;
    return false;
  }
  else {
    DEBUG_PRINTLN("Recovered registration details:");
    DEBUG_PRINTLN(full_name);
    DEBUG_PRINTLN(phone_number);
    return true;
  }
}

// if registration occured, existance flag may indicate:
// CODE_OK: registration passed
// CODE_NOT_OK: registration failed,
// CODE_UNKNOWN: registration not yet attempted
// Sadly, this semantic was not implemented in wifi parameters
// TODO: you know,
boolean eeprom_checkRegistrationExist() {
  EEPROM.begin(512);
  char exist[2 + 1];
  EEPROM.get(REGISTRATION_STATUS_EEPROM_LOCATION, exist);
  EEPROM.end();
  String existance = String(exist);
  return (existance == String(CODE_OK) || existance == String(CODE_NOT_OK) || existance == String(CODE_UNKNOWN));
}

int eeprom_getRegistrationStatus() {
  EEPROM.begin(512);
  char exist[2 + 1];
  EEPROM.get(REGISTRATION_STATUS_EEPROM_LOCATION, exist);
  EEPROM.end();
  String existance = String(exist);
  if (existance == String(CODE_OK)) {
    return REG_STATUS_PASSED;
  }
  else if (existance == String(CODE_NOT_OK)) {
    return REG_STATUS_FAILED;
  }
  else {
    return REG_STATUS_UNKNOWN;
  }
}

void eeprom_setup() {
  pinMode(CLEAR_EEPROM_CREDENTIALS_PIN, INPUT_PULLUP);
  if (!digitalRead(CLEAR_EEPROM_CREDENTIALS_PIN)) {
    DEBUG_PRINTLN("reset pin triggered, clearing eeprom");
    eeprom_clear(true);
  }
  eeprom_loadCredentials(); // Load WLAN credentials from EEPROM
}

int* eeprom_getStoredReports() {
  EEPROM.begin(512);
  EEPROM.get(REPORT_HISTORY_EEPROM_LOCATION, history);
  EEPROM.end();
  return history;
}

int* eeprom_storeReport(int val) {
  EEPROM.begin(512);
  EEPROM.get(REPORT_HISTORY_EEPROM_LOCATION, history);
  for(int i = 0; i < (REPORT_HISTORY_LENGTH-1); ++i) {
    history[i*2] = history[(i+1)*2];
    history[i*2+1] = history[(i+1)*2+1];
  }
  DEBUG_PRINT("result"); DEBUG_PRINT(history[(REPORT_HISTORY_LENGTH-1)*2] + 1); 
  DEBUG_PRINT("("); DEBUG_PRINT((REPORT_HISTORY_LENGTH-1)*2); DEBUG_PRINT(")"); 
  DEBUG_PRINT("="); DEBUG_PRINT(val);
  DEBUG_PRINTLN();
  // counter
  history[(REPORT_HISTORY_LENGTH-1)*2] = history[(REPORT_HISTORY_LENGTH-2)*2] + 1;
  // value
  history[(REPORT_HISTORY_LENGTH-1)*2 + 1] = val;
  EEPROM.put(REPORT_HISTORY_EEPROM_LOCATION, history);
  EEPROM.commit();
  EEPROM.end();
  return history;
}

void eeprom_setNewRollFlag(boolean flag) {
  EEPROM.begin(512);
  EEPROM.put(NEW_ROLL_FLAG_EEPROM_LOCATION, flag);
  EEPROM.commit();
  EEPROM.end();
}

boolean eeprom_getNewRollFlag() {
  EEPROM.begin(512);
  byte result = 0;
  EEPROM.get(NEW_ROLL_FLAG_EEPROM_LOCATION, result);
  EEPROM.end();
  if(result > 1 || result < 0) {
    eeprom_setNewRollFlag(false);
    result = 0;
  }
  return (result == 1);
}

//
//void eeprom_testHistory() {
//  DEBUG_PRINTLN(REPORT_HISTORY_EEPROM_LOCATION);
//  int* h = eeprom_getStoredReports();
//  for (int i = 0; i < REPORT_HISTORY_LENGTH * 2; ++i) {
//    DEBUG_PRINT(h[i]);
//    DEBUG_PRINT(' ');
//  }
//  DEBUG_PRINTLN();
//  for (int i = 0; i < 10; ++i) {
//    int v = random(0, 100);
//    int* h = eeprom_storeReport(v);
//    for (int i = 0; i < REPORT_HISTORY_LENGTH * 2; ++i) {
//      DEBUG_PRINT(h[i]);
//      DEBUG_PRINT(' ');
//    }
//    DEBUG_PRINTLN();
//  }
//  delay(1000);
//}

