#include "I2Cdev.h"
#include "MPU6050.h"
#include "Wire.h"


#define TURN_STATE_COUNT 8
#define SENSOR_TOLERANCE 1000
#define MAX_TURN_SKIP 2

#define RESET_ENABLE_PIN D0
#define GY521_ENABLE_PIN D8

MPU6050 accelgyro;

int16_t ax, ay, az;
int16_t gx, gy, gz;

boolean blinkState = false;
int lastTurnState = -1;
int curr_turn_state = -1;
int turns_acc = 0;
int curr_round = 0;

int turn_states[TURN_STATE_COUNT][2] = {
  {0, 15000},
  {9600, 12500},
  {17300, -2500},
  {12500, -13200},
  {1200, -18500},
  { -10000, -14000},
  { -15600, -1100},
  { -11200, 10000},
};

void setup() {
  delay(500);
  Serial.begin(9600);
  pinMode(BUILTIN_LED, OUTPUT);
  pinMode(RESET_ENABLE_PIN, WAKEUP_PULLUP);
  pinMode(GY521_ENABLE_PIN, OUTPUT);
  digitalWrite(GY521_ENABLE_PIN, HIGH);
  delay(100);
  setup_acc();
}

void loop() {
  //accelgyro.getMotion6(&ax, &ay, &az, &gx, &gy, &gz);
  accelgyro.getAcceleration(&ax, &ay, &az);
  if (Serial.available()) {
    Serial.print(ax); Serial.print("\t");
    Serial.print(az); Serial.print("\t");
    Serial.println();
    while (Serial.available()) {
      Serial.read();
      delay(1);
    }
  }
  update_turn_state();
  blinkState = !blinkState;
  digitalWrite(BUILTIN_LED, blinkState);
}

void setup_acc() {
  Serial.println("Initializing I2C devices...");
  Wire.begin();
  accelgyro.initialize();

  // verify connection
  Serial.println("Testing device connections...");
  Serial.println(accelgyro.testConnection() ? "MPU6050 connection successful" : "MPU6050 connection failed");
}

void update_turn_state() {
  boolean printout = false;
  for (int t = 0; t < TURN_STATE_COUNT; t++) {
    if (abs(ax - turn_states[t][0]) < SENSOR_TOLERANCE && abs(az - turn_states[t][1]) < SENSOR_TOLERANCE && lastTurnState != t) {
      Serial.print(t); Serial.print(' ');
      printout = true;
    }
  }
  if (printout) {
    Serial.println();
  }
}

