#define TURN_STATE_COUNT 8
#define SENSOR_TOLERANCE 1000
#define MAX_TURN_SKIP 2

#define GY521_ENABLE_PIN D8

#define STABILITY_QUALIFICATION_DURATION 7000
#define STABILITY_TOLERANCE 1500

MPU6050 accelgyro;

int16_t ax, ay, az;
int16_t prev_ax, prev_ay, prev_az;
int16_t gx, gy, gz;

int curr_turn_state = -1;
int turns_acc = 0;
int curr_round = 0;
long stability_start_tstamp = -1;

int turn_states[TURN_STATE_COUNT][2] = {
  {0, 15000},
  {9600, 12500},
  {17300, -2500},
  {12500, -13200},
  {1200, -18500},
  { -10000, -14000},
  { -15600, -1100},
  { -11200, 10000},
};


void acc_setup() {
  Serial.println("Initializing I2C devices...");
  Wire.begin();
  accelgyro.initialize();

  // verify connection
  Serial.println("Testing device connections...");
  Serial.println(accelgyro.testConnection() ? "MPU6050 connection successful" : "MPU6050 connection failed");
}

boolean acc_is_stable() {
  accelgyro.getAcceleration(&ax, &ay, &az);
  if(abs(ax - prev_ax) < STABILITY_TOLERANCE && 
     abs(ay - prev_ay) < STABILITY_TOLERANCE && 
     abs(az - prev_az) < STABILITY_TOLERANCE) {

    if(stability_start_tstamp < 0) {
      stability_start_tstamp = millis();
    }
    else {
      return (millis() - stability_start_tstamp > STABILITY_QUALIFICATION_DURATION);
    }
  }
  else {
    stability_start_tstamp = -1;
  }
  prev_ax = ax;
  prev_ay = ay;
  prev_az = az;
  return false;
}

int acc_get_turn_state() {
  int result = -1;
//  boolean printout = false;
  for (int t = 0; t < TURN_STATE_COUNT; t++) {
    if (abs(ax - turn_states[t][0]) < SENSOR_TOLERANCE && abs(az - turn_states[t][1]) < SENSOR_TOLERANCE) {
      result = t;
      //Serial.print(curr_turn_state); Serial.print(' ');
      //printout = true;
    }
  }
//  if (printout) {
//    Serial.println();
//  }
  return result;
}

int shake_acc = 0;

int acc_count_turns() {
  int result = 0;
  curr_round = 0;
  stability_start_tstamp = -1;
  while(!acc_is_stable()) {
    accelgyro.getAcceleration(&ax, &ay, &az);
    float force = sqrt(ax*ax + ay*ay + az*az);
    shake_acc = shake_acc + abs(force-15000)/5 - 1500;
    if(shake_acc < 0) {
      shake_acc = 0;
    }
    Serial.print(force);
    Serial.print("\t");
    Serial.println(shake_acc);
    int turn_state = acc_get_turn_state();
    // are we in a valid turn state?
    if(turn_state >= 0) { 
      // has there been a change in the turn state?
      if(turn_state != curr_turn_state) {
        // is this the second valid detection onwards?
        if(curr_turn_state > -1) {
          // is this a part of a smooth movement?
          int step_count = 
            min(abs(turn_state - curr_turn_state), 
            min(abs(turn_state - curr_turn_state + TURN_STATE_COUNT),
                abs(turn_state - curr_turn_state - TURN_STATE_COUNT)));
          if(step_count <= MAX_TURN_SKIP) {
            // update accumulator
            turns_acc += step_count;
            // Serial.println(turns_acc);
            if(abs(turns_acc) >= (TURN_STATE_COUNT)) {
              curr_round++;
              Serial.println(curr_round);
              result = curr_round;
              if(turns_acc > 0) {
                turns_acc -= TURN_STATE_COUNT;
              }
              else {
                turns_acc += TURN_STATE_COUNT;
              }
            }
          }
        }
        curr_turn_state = turn_state;
      }
    }  
  }
  return result;
}

